<?php

/* Les lignes suivantes simulent le .htaccess */
if (preg_match('/\.(?:css|png|jpg|jpeg|gif)$/', $_SERVER["REQUEST_URI"])) {
    return false;    // serve the requested resource as-is.
}

/* Ajoute la racine a` l'include path */
set_include_path(__DIR__ . PATH_SEPARATOR . get_include_path());

include 'config/config.php';
foreach ($config['core_classes'] as $classname) {
    require "core/$classname.php";
}

function __autoload($class_name) {
    include "classes/$class_name.php";
}

class FrontController {

    private static function get_path_elements() {
        if (!isset($_SERVER['PATH_INFO'])) {
            return null;
        }
        $path_info = $_SERVER['PATH_INFO'];
        $path_elements = explode('/', $path_info);
        array_shift($path_elements);
        return $path_elements;
    }

    private static function path_contains_controller_name($path_elements) {
        return $path_elements !== null && count($path_elements) >= 1 && ctype_alnum($path_elements[0]);
    }

    private static function path_contains_method_name($path_elements) {
        return $path_elements !== null && count($path_elements) >= 2;
    }

    private static function get_controller_name($path_elements) {
        global $config;

        return (self::path_contains_controller_name($path_elements)) ? $path_elements[0] : $config['default_controller'];
    }

    private static function get_method_name($path_elements) {
        global $config;

        return (self::path_contains_method_name($path_elements)) ? $path_elements[1] : $config['default_method'];
    }

    private static function create_controller($controller_name) {
        $controller_classname = ucfirst(strtolower($controller_name));
        $controller_filename = 'controllers/' . $controller_classname . '.php';

        if (!file_exists($controller_filename))
            self::generate_error_404();
        require $controller_filename;

        if (!class_exists($controller_classname))
            self::generate_error_404();

        return new $controller_classname();
    }

    private static function call_controller_method($controller, $method_name) {

        $reflectionObject = new ReflectionObject($controller);
        if (!$reflectionObject->hasMethod($method_name))
            self::generate_error_404();

        $reflectionMethod = $reflectionObject->getMethod($method_name);
        $reflectionMethod->invoke($controller);
    }

    private static function generate_error_404() {
        header("HTTP/1.0 404 Not Found");
        exit;
    }

    static function main() {

        $path_elements = self::get_path_elements();
        $controller_name = self::get_controller_name($path_elements);
        $method_name = self::get_method_name($path_elements);
        $controller = self::create_controller($controller_name);


        self::call_controller_method($controller, $method_name);
    }

}

FrontController::main();
